import { interval, roat } from './data.structures';
/**
 * Complement of `intervals` bounded to `boundaries`. Convert space between two consecutive intervals into interval.
 * Keeps extra object properties on `boundaries`.
 * Curried function. Accept array of intervals. Doesn't mutate input. Output keeps input's structure.
 *
 * boundaries | interval(s) | result
 * --- | --- | ---
 * { start: 0, end: 10} | { start: 3, end: 7 } | [{ start: 0, end: 3 }, { start: 7, end: 10 }]
 * { start: 0, end: 10} | [{ start: 2, end: 4 }, { start: 7, end: 8 }] | [{ start: 0, end: 2 }, { start: 4, end: 7 }, { start: 8, end: 10 }]
 *
 * @param boundaries arg1: interval defining boundaries for the complement computation.
 * @param intervals arg2: one interval or array of intervals that complement the result.
 * @returns array of intervals.
 */
export declare function complement<D extends interval, T extends interval>(boundaries: D, interv: T | roat<T>): D[];
export declare function complement<D extends interval, T extends interval>(boundaries: D): (interv: T | roat<T>) => D[];
/**
 * Test if `intervalA` overlaps with `intervalB`.
 *
 * Curried function. Accept array of intervals.
 *
 * intervalA | intervalB | result
 * --- | --- | ---
 * { start: 0, end: 10} | { start: 3, end: 7 } | true
 * { start: 0, end: 5} | { start: 5, end: 7 } | false
 * { start: 5, end: 10} | [{ start: 0, end: 4 }, { start: 7, end: 8 }] | true
 *
 * @param intervalA arg1: interval or array of intervals
 * @param intervalB arg2: interval or array of intervals
 * @returns true if overlaps
 */
export declare function isOverlapping(intervalA: interval | roat<interval>, intervalB: interval | roat<interval>): boolean;
export declare function isOverlapping(intervalA: interval | roat<interval>): (intervalB: interval | roat<interval>) => boolean;
/**
 * Test if `intervalA` is adjacent to (meets) `intervalB`.
 *
 * Curried function.
 *
 * intervalA | intervalB | result
 * --- | --- | ---
 * { start: 0, end: 10} | { start: 3, end: 7 } | false
 * { start: 0, end: 5} | { start: 5, end: 7 } | true
 *
 * @param intervalA arg1: interval
 * @param intervalB arg2: interval
 * @returns true if adjacent
 */
export declare function isMeeting(intervalA: interval, intervalB: interval): boolean;
export declare function isMeeting(intervalA: interval): (intervalB: interval) => boolean;
/**
 * Test if `intervalA` is before or adjacent `intervalB`.
 *
 * Curried function.
 *
 * intervalA | intervalB | result
 * --- | --- | ---
 * { start: 0, end: 2} | { start: 3, end: 7 } | true
 * { start: 0, end: 5} | { start: 3, end: 7 } | false
 *
 * @param intervalA arg1: interval
 * @param intervalB arg2: interval
 * @returns true if before
 */
export declare function isBefore(intervalA: interval, intervalB: interval): boolean;
export declare function isBefore(intervalA: interval): (intervalB: interval) => boolean;
/**
 * Test if `intervalA` is after or adjacent `intervalB`.
 *
 * Curried function.
 *
 * intervalA | intervalB | result
 * --- | --- | ---
 * { start: 5, end: 10} | { start: 3, end: 4 } | true
 * { start: 5, end: 10} | { start: 3, end: 6 } | false
 *
 * @param intervalA arg1: interval
 * @param intervalB arg2: interval
 * @returns true if after
 */
export declare function isAfter(intervalA: interval, intervalB: interval): boolean;
export declare function isAfter(intervalA: interval): (intervalB: interval) => boolean;
/**
 * Test if `intervalA` and `intervalB` share the same starting point.
 *
 * Curried function.
 *
 * intervalA | intervalB | result
 * --- | --- | ---
 * { start: 5, end: 10} | { start: 5, end: 4 } | true
 * { start: 5, end: 10} | { start: 0, end: 10 } | false
 *
 * @param intervalA arg1: interval
 * @param intervalB arg2: interval
 * @returns true if same starting point
 */
export declare function isStarting(intervalA: interval, intervalB: interval): boolean;
export declare function isStarting(intervalA: interval): (intervalB: interval) => boolean;
/**
 * Test if `intervalA` and `intervalB` share the same ending point.
 *
 * Curried function.
 *
 * intervalA | intervalB | result
 * --- | --- | ---
 * { start: 5, end: 10} | { start: 0, end: 10 } | true
 * { start: 5, end: 10} | { start: 5, end: 7 } | false
 *
 * @param intervalA arg1: interval
 * @param intervalB arg2: interval
 * @returns true if same ending point
 */
export declare function isEnding(intervalA: interval, intervalB: interval): boolean;
export declare function isEnding(intervalA: interval): (intervalB: interval) => boolean;
/**
 * Test if `intervalA` occurs in `intervalB`. `intervalsB` act as boundaries. Can share starting and/or ending point.
 *
 * Curried function.
 *
 * intervalA | intervalB | result
 * --- | --- | ---
 * { start: 2, end: 6} | { start: 0, end: 10 } | true
 * { start: 5, end: 10} | { start: 0, end: 10 } | true
 * { start: 5, end: 10} | { start: 0, end: 9 } | false
 *
 * @param intervalA arg1: interval
 * @param intervalB arg2: interval
 * @returns true if `intervalA` occurs in `intervalB`
 */
export declare function isDuring(intervalA: interval, intervalB: interval): boolean;
export declare function isDuring(intervalA: interval): (intervalB: interval) => boolean;
/**
 * Test if `intervalA` is equivalent to `intervalB`.
 *
 * Curried function.
 *
 * intervalA | intervalB | result
 * --- | --- | ---
 * { start: 5, end: 10} | { start: 5, end: 10 } | true
 * { start: 5, end: 10} | { start: 0, end: 10 } | false
 *
 * @param intervalA arg1: interval
 * @param intervalB arg2: interval
 * @returns true if equivalent
 */
export declare function isEqual(intervalA: interval, intervalB: interval): boolean;
export declare function isEqual(intervalA: interval): (intervalB: interval) => boolean;
/**
 * Simplification of `intervals`. Unify touching or overlapping intervals.
 *
 * Doesn't mutate input. Output keeps input's structure.
 *
 * | intervals A | result |
 * | ----------- | ------ |
 * | [{ start: 3, end: 9 }, { start: 9, end: 13 }, { start: 11, end: 14 }] | [{ start: 3, end: 14 }] |
 *
 * @param intervalA
 */
export declare function simplify<T extends interval>(intervalA: roat<T>): T[];
/**
 * Union of `intervals`.
 *
 * Curried function. Accept array of intervals. Doesn't mutate input. Output keeps input's structure.
 *
 * interval(s) A | interval(s) B | result
 * --- | --- | ---
 * { start: 0, end: 4} | { start: 3, end: 7 } | [{ start: 0, end: 7 }]
 * { start: 0, end: 4} | [{ start: 3, end: 7 }, { start: 9, end: 11 }] | [{ start: 0, end: 7 }, { start: 9, end: 11 }]
 *
 * @param intervalA arg1: one interval or array of intervals
 * @param intervalB arg2: one interval or array of intervals
 * @returns union of `arg1` and `arg2`
 */
export declare function unify<T extends interval>(intervalA: T | roat<T>, intervalB: T | roat<T>): T[];
export declare function unify<T extends interval>(intervalA: T | roat<T>): (intervalB: T | roat<T>) => T[];
/**
 * Intersection of `intervals`. Does not simplify result. Keeps extra object properties on `intervalB`.
 *
 * `interalA` and `interalB` can have different structure.
 * Curried function. Accept array of intervals. Doesn't mutate input. Output keeps `intervalB` structure.
 *
 * interval(s) A | interval(s) B | result
 * --- | --- | ---
 * { from: 0, to: 4 } | { start: 3, end: 7, foo: 'bar' } | [{ start: 3, end: 4, foo: 'bar' }]
 * { start: 0, end: 10 } | [{ start: 2, end: 5}, { start: 5, end: 8}] | [{ start: 2, end: 5 }, { start: 5, end: 8 }]
 * [{ start: 0, end: 4 }, { start: 8, end: 11 }] | [{ start: 2, end: 9 }, { start: 10, end: 13 }] | [{ start: 2, end: 4 }, { start: 8, end: 9 }, { start: 10, end: 11 }]
 *
 * @param intervalA arg1: one interval or array of intervals
 * @param intervalB arg2: one interval or array of intervals
 * @returns intersection of `arg1` and `arg2`
 */
export declare function intersect<T extends interval>(intervalA: interval | roat<interval>, intervalB: T | roat<T>): T[];
export declare function intersect<T extends interval>(intervalA: interval | roat<interval>): (intervalB: T | roat<T>) => T[];
/**
 * Merge extra properties of all intervals inside `intervals`, when overlapping, with provided function `mergeFn`.
 * Can also be used to generate an array of intervals without overlaps
 *
 * Curried function. Doesn't mutate input. Output keeps input's structure.
 *
 * parameter | value
 * --- | ---
 * mergeFn | `(a, b) => {...a, data: a.data + b.data }`
 * intervals | `[{ start: 0, end: 10, data: 5 }, { start: 4, end: 7, data: 100 }]`
 * result | `[{ start: 0, end: 4, data: 5 }, { start: 4, end: 7, data: 105 }, { start: 7, end: 10, data: 5 }]`
 * @param mergeFn arg1: function to merge extra properties of overlapping intervals
 * @param intervals arg2: intervals with extra properties.
 */
export declare function merge<T extends interval>(mergeFn: (ints: T[]) => T, intervals: roat<T>): T[];
export declare function merge<T extends interval>(mergeFn: (ints: T[]) => T): (intervals: roat<T>) => T[];
/**
 * Subtact `intervalA` with `intervalB`. `intervalB` act as a mask.
 * Keeps extra object properties on `intervalA`.
 *
 * Curried function. Accept array of intervals. Doesn't mutate input. Output keeps input's structure.
 *
 * interval(s) A | interval(s) B | result
 * --- | --- | ---
 * { start: 0, end: 4 } | { start: 3, end: 7 } | [{ start: 0, end: 3 }]
 * [{ start: 0, end: 4 }, { start: 8, end: 11 }] | [{ start: 2, end: 9 }, { start: 10, end: 13 }] | [{ start: 0, end: 2 }, { start: 9, end: 10 }]
 *
 * @param intervalA arg1: one interval or array of intervals
 * @param intervalB arg2: one interval or array of intervals
 * @returns intersection of `arg1` and `arg2`
 */
export declare function substract<D extends interval, T extends interval>(intervalA: D | roat<D>, intervalB: T | roat<T>): D[];
export declare function substract<D extends interval, T extends interval>(intervalA: D | roat<D>): (intervalB: T | roat<T>) => D[];
